#!/bin/csh -f

# notes:
# (1) not checking for format inconsistency like 1234 vs ACGT coding (will cause problems: most, if not all, would be non-biallelic and discarded)
# (2) assuming the third column [could change] contains the actual alleles for both sets
# (3) not checking for singleton or monomorphic SNPs: could contain them (2009-12-18, now added)
# (4) programs called: 
#       (A) catHaplotypes.pl
#       (B) HapNotBiallelic
#       (C) h2h.pl

set ahf = $1
set asf = $2
set bhf = $3
set bsf = $4
set pre = $5
set col = $6
set minMAC = $7

if ($pre == "") then
  set pre = catHapKeepBiallelic.out
endif
if ($col == "") then
  set col = 3
endif
if ($minMAC == "") then
  echo "parameter minMAC (min Minor Allele Count) not specified, default = 0 (all markers regardless of minor allele count)"  
  set minMAC = 0
endif

echo "catHaplotypesKeepBiallelic: concatenate two sets of haplotypes and keep only the biallelic ones"
echo "(c) 2007-2009 Yun li \@ yunli@bios.unc.edu"
echo ""

if ($ahf == "" || $asf == "" || $bhf == "" || $bsf == "") then
  echo "usage: catHaplotypesKeepBiallelic.csh ahf asf bhf bsf [prefix] [hap_start_field>=1, defaul=3] [minMAC, defaul=0]"
  echo "exiting"
  echo ""
  exit
endif


# (1) concatenate haplotypes to retain SNPs common to both sets
#       regardless of # alleles, format etc
#
catHaplotypes.pl -ah $ahf -as $asf -bh $bhf -bs $bsf -o $pre.all > $pre.cathap.log
rm -f $pre.cathap.log

# (2) remove SNPs that have >2 alleles
#       could due to 1234 and ACGT coding
#
# (2-1) find non-biallelic SNPs and in the same order of input SNP
HapNotBiallelic -h $pre.all.hap -s $pre.all.snps > $pre.hapNotBiallelic.log
@ lines = `grep -n "The following markers have more than two alleles" $pre.hapNotBiallelic.log | cut -f1 -d ':'`
sed 1,${lines}d $pre.hapNotBiallelic.log > $pre.nonBiallelic.snps
rm -f $pre.hapNotBiallelic.log
cat $pre.nonBiallelic.snps $pre.all.snps | sort | uniq -c | awk '$1 == 1' | awk '{print $2}' > $pre.biallelic.snps
rm -f $pre.nonBiallelic.snps
extractor $pre.biallelic.snps < $pre.all.snps > $pre.final.snps
rm -f $pre.biallelic.snps

# (2-2) extract the set of biallelic SNPs from the common set
h2h.pl -h $pre.all.hap -s $pre.all.snps -r $pre.final.snps -col $col -o $pre > $pre.h2h.log
rm -f $pre.h2h.log
rm -f $pre.all.*
rm -f $pre.final.snps

# (3) remove SNPs that are monomorphic or singleton
if ($minMAC > 0) then
  # (3-0) count total number of alleles/haplotypes
  @ minMAC --
  @ nhap = `wc -l < $pre.hap`
  @ maf = `echo "$minMAC *100000 / $nhap" | bc`
  @ maf ++ 
  @ minMAC ++
  @ umaf = `echo "$minMAC *100000 / $nhap" | bc`
  while ($maf >= $umaf)
    @ maf --
  end
  @ minMAC --

  # (3-1) h2g.pl to get fake .ped
  h2g.pl -h $pre.hap -o $pre.ped > $pre.h2g.log
  rm -f $pre.h2g.log

  # (3-2) get .dat
  sed -e 's/^/M /g' $pre.snps > $pre.dat

  # (3-2) run new_fugue to get frequencies
  new_fugue -d $pre.dat -p $pre.ped --alleles -o $pre > $pre.nf.log 
  rm -f $pre.nf.log

  # (3-3) get list of SNPs with MAC >= $minMAC
  awk '{$2=$2*1e5;print}' $pre.maf | awk -v co=$maf '$2 > co' | awk '{print $1}' > $pre.common.input.snps
  
  # (3-4) clean files
  rm -f $pre.ped $pre.dat $pre.freq 
  # rm -f $pre.maf #keep $pre.maf

  # (3-5) extract common snps
  h2h.pl -h $pre.hap -s $pre.snps -r $pre.common.input.snps -col $col -o $pre.common > $pre.common.h2h.log
  rm -f $pre.common.h2h.log
endif
 
