#!/usr/bin/perl -w
use strict;
use Getopt::Long;


###############################################################
##################### DOCUMENTATION ###########################

## Author: Yun Li
## Date: Nov 2006
## Goal: Get .ped format files from Haplotypes (ONE field only) and ID's
## Location: /home/ylwtx/mach1/OAAT_TEST

## Input Parameters
##

## Input Files
##

## Output Files

## changelog
## 0.0.3
##	(1) if ID/actually the first fields not provided, first see if the haplotype file contain more than one field
##		if so, use the first field as family and individual ID

################# END OF DOCUMENTATION ########################
###############################################################

print ("\nh2g.pl 0.0.3: convert haplotype file to genotepe file\n");
print ("Yun Li (2007-2009), comments to yunli\@med.unc.edu\n\n\n");

my $MIN_NUM_OPTS = 2;

if($#ARGV<($MIN_NUM_OPTS*2 - 1))
{
	&usage();
	die "Failed to parse parameters\n\n";
}


my %opts = ();

	# Default Optional Options
$opts{id} = 0;

Getopt::Long::GetOptions(\%opts,qw(
	h=s
	id=s
	o=s
)) || die "Failed to parse options\n\n";

&printPar();


&main();



# begin sub

sub usage
{
	print "\n";
	print "Usage:\n\t";
	print "-h\t\tHaplotype File(Required, last field for haplotypes)\n\t";
	print "-id\t\tID Column File(Optional, expect at least 5 fields)\n\t";
	print "-o\t\tOutput File(Required)\n\t";
	print "\n";
}


sub printPar
{
	print "\n";
	print "Parameters in Effect:\n";
	print "\tHaplotype File \n\t\t-h '$opts{h}'\n";
	print "\tID Column File \n\t\t-id '$opts{id}'\n";
	print "\tOutput \n\t\t-o '$opts{o}'\n";
	print "\n";
}




sub rmXtraFiles
{
	if(-e "tmpt.txt")
	{
		system "/bin/rm -r tmpt.txt";
	}
}



sub swap
{
	my $tmp = $_[0];
	$_[0] = $_[1];
	$_[1] = $tmp;
}


sub main
{
	
	&no_spaced_version();
}

sub spaced_version
{
	open(OUT, ">$opts{o}") || die "Can't create file $opts{o}\n\n";
	close(OUT);
	
	if($opts{id})
	{
		open(ID, $opts{id}) || die "Can't open file $opts{id}\n\n";
	}
	
	open(OUT, ">>$opts{o}") || die "Can't append file $opts{o}\n\n";
	open(IN, $opts{h}) || die "Can't open file $opts{i}\n\n";
	my $i = 0;
	while(my $line1=<IN>)
	{
		$i ++;
		chomp($line1);
		$line1 =~ s/^\s+//;
                my @lineArr1 = (split(/\s+/,$line1));
                $line1 = $lineArr1[$#lineArr1];
		my $line2 = <IN>;
		chomp($line2);
		$line2 =~ s/^\s+//;
                my @lineArr2 = (split(/\s+/,$line2));
                $line2 = $lineArr2[$#lineArr2];
		
		if(!$opts{id})
		{
			if (scalar(@lineArr1) == 1)
			{
				if($i<=9)
				{
					print OUT "H_000$i H_000$i 0 0 2 ";
				}
				else
				{
					print OUT "H_00$i H_00$i 0 0 2 ";
				}
			}
			else
			{
				print OUT "$lineArr1[0] $lineArr1[0] 0 0 2 ";
			}
		}
		else
		{
			my $crt_ID = <ID>;
			chomp($crt_ID);
			print OUT "$crt_ID ";
		}
		
		my @line1Arr = (split(/\s+/,$line1));
		my @line2Arr = (split(/\s+/,$line2));
		for(my $j=0; $j<$#line1Arr;$j++)
		{
			print OUT "$line1Arr[$j]/$line2Arr[$j] ";
		}
		for(my $j=$#line1Arr; $j<=$#line1Arr;$j++)
		{
			print OUT "$line1Arr[$j]/$line2Arr[$j]\n";
		}
	}
	close(IN);
	close(OUT);
	if($opts{id})
	{
		close(ID);
	}
}


sub no_spaced_version
{
	open(OUT, ">$opts{o}") || die "Can't create file $opts{o}\n\n";
	close(OUT);
	
	if($opts{id})
	{
		open(ID, $opts{id}) || die "Can't open file $opts{id}\n\n";
	}
	
	open(OUT, ">>$opts{o}") || die "Can't append file $opts{o}\n\n";
	open(IN, $opts{h}) || die "Can't open file $opts{i}\n\n";
	my $i = 0;
	while(my $line1=<IN>)
	{
		$i ++;
		chomp($line1);
		$line1 =~ s/^\s+//;
		my @L1arr = (split(/\s+/,$line1));
		$line1 = $L1arr[$#L1arr];
		my $line2 = <IN>;
		chomp($line2);
		$line2 =~ s/^\s+//;
		my @L2arr = (split(/\s+/,$line2));
		$line2 = $L2arr[$#L2arr];
		
		if(!$opts{id})
		{
			if($i<=9)
			{
				print OUT "H_000$i H_000$i 0 0 2 ";
			}
			else
			{
				print OUT "H_00$i H_00$i 0 0 2 ";
			}
		}
		else
		{
			my $crt_ID = <ID>;
			chomp($crt_ID);
			print OUT "$crt_ID ";
		}
		
		my @line1Arr = (split(//,$line1));
		my @line2Arr = (split(//,$line2));
		for(my $j=0; $j<$#line1Arr;$j++)
		{
			print OUT "$line1Arr[$j]/$line2Arr[$j] ";
		}
		for(my $j=$#line1Arr; $j<=$#line1Arr;$j++)
		{
			print OUT "$line1Arr[$j]/$line2Arr[$j]\n";
		}
	}
	close(IN);
	close(OUT);
	if($opts{id})
	{
		close(ID);
	}

}
